<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 15/04/19
 * Time: 16.06
 */

namespace FiloBlu\Hotfix\Plugin\Model\Rule\Condition;

use Magento\Catalog\Model\ResourceModel\Product as ProductResource;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\ProductFactory;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;
use Magento\Catalog\Model\ProductRepository;

class Product
{
    protected $productResource;
    protected $categoryRepository;
    protected $productFactory;
    protected $productMetadata;
    protected $configurable;
    protected $productRepository;

    /**
     * Product constructor.
     * @param CategoryRepositoryInterface $categoryRepository
     * @param ProductResource $productResource
     * @param ProductFactory $productFactory
     * @param ProductMetadataInterface $productMetadata
     * @param Configurable $configurable
     * @param ProductRepository $productRepository
     */
    public function __construct(CategoryRepositoryInterface $categoryRepository,
                                ProductResource $productResource,
                                ProductFactory $productFactory,
                                ProductMetadataInterface $productMetadata,
                                Configurable $configurable,
                                ProductRepository $productRepository)
    {
        $this->categoryRepository = $categoryRepository;
        $this->productResource = $productResource;
        $this->productFactory = $productFactory;
        $this->productMetadata = $productMetadata;
        $this->configurable = $configurable;
        $this->productRepository = $productRepository;

    }

    /**
     * @return string
     */
    public function getVersion()
    {
        return $this->productMetadata->getVersion();
    }

    /**
     * @return bool
     */
    public function checkIfToApplyPlugin()
    {
        if ($this->getVersion() < "2.3.1")
        {
            return true;
        }
        else
        {
            return false;
        }
    }


    /**
     * @param \Magento\SalesRule\Model\Rule\Condition\Product $product
     * @param $result
     * @param \Magento\Framework\Model\AbstractModel $model
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function aroundValidate( \Magento\SalesRule\Model\Rule\Condition\Product $product, $result, \Magento\Framework\Model\AbstractModel $model)
    {

        if($this->checkIfToApplyPlugin() && $product->getAttribute() == "category_ids")
        {

            //as written in \Magento\SalesRule\Model\Rule\Condition\Product :: validate
            ////@todo reimplement this method when is fixed MAGETWO-5713
            //MAGETWO-5713 merged into 2.3.1 . For more info see https://github.com/magento/magento2/commit/3de5d0e6e6df9e5d860fbdd1750347d7d606a204
            $myResult = false;
            $operator = $product->getOperator();
            $category = $product->getValueParsed();
            $productId = $model->getProductId();
            $myProduct = $this->productRepository->getById($productId);

            $myProductCategory = $this->getCategoryIdsFromProduct($myProduct);

            //use is one of o is not one of for contains or not contains logic
            $operator = str_replace("{}","()",$operator);

            switch ($operator)
            {
                case "==":
                    if(is_array($category))
                    {
                        if (count(array_diff($category , $myProductCategory))==0)
                        {
                            $myResult = true;
                        }
                    }
                    else
                    {
                        foreach ($myProductCategory as $item)
                        {
                            if($category == $item)
                            {
                                $myResult = true;
                            }
                        }
                    }
                    break;

                case "!=":
                    if(is_array($category))
                    {
                        if (count(array_diff($category , $myProductCategory))==count($category))
                        {
                            $myResult = true;
                        }
                    }
                    else
                    {
                        if (!in_array($category , $myProductCategory))
                        {
                            $myResult = true;
                        }
                    }
                    break;

                case "()":
                    if (count(array_diff($category , $myProductCategory))<count($category))
                    {
                        $myResult = true;
                    }
                    break;

                case "!()":
                    if(is_array($category))
                    {
                        if (count(array_diff($category , $myProductCategory))==count($category))
                        {
                            $myResult = true;
                        }
                    }
                    else
                    {
                        if (!in_array($category, $myProductCategory))
                        {
                            $myResult = true;
                        }
                    }
                    break;
            }
            return $myResult;
        }
        else
        {
            return $result($model);
        }
    }

    /**
     * @param $myProduct
     * @return array
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCategoryIdsFromProduct($myProduct)
    {
        $myProductCategory = [];
        if($myProduct->getTypeId() == "configurable")
        {
            $myProductCategory = $myProduct->getCategoryIds();
        }
        else if($myProduct->getTypeId() == "simple")
        {
            //get Category from parent configurable
            $configurableIds = $this->configurable->getParentIdsByChild($myProduct->getId());
            $catArray = [];
            foreach ($configurableIds as $value)
            {
                $configurable = $this->productRepository->getById($value);
                $catArray = array_merge($catArray,$this->productResource->getCategoryIds($configurable));
            }

            //get Category from simple
            if(count($this->productResource->getCategoryIds($myProduct))>0)
            {
                $catArray  = array_merge($catArray,$this->productResource->getCategoryIds($myProduct));
            }


            $myProductCategory = $catArray;
        }

        return $myProductCategory;
    }

}